@@ -34,6 +34,7 @@
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <string.h>
+#include <limits.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -245,7 +246,8 @@ int is_raw_80211(const char *iface)
     struct ifreq ifr;
     struct sockaddr sa;
 
-    strncpy(ifr.ifr_name, iface, IFNAMSIZ);
+    strncpy(ifr.ifr_name, iface, IFNAMSIZ-1);
+    ifr.ifr_name[IFNAMSIZ-1]='\0';
 
     if (skfd && ioctl(skfd, SIOCGIFHWADDR, &ifr) < 0) {
         di_warning("Unable to retrieve interface type.");
@@ -267,7 +269,7 @@ int is_raw_80211(const char *iface)
 
 #if defined(__s390__)
 // Layer 3 qeth on s390(x) cannot do arping to test gateway reachability.
-int is_layer3_qeth(const char *iface)
+int is_layer3_qeth(const struct netcfg_interface *interface)
 {
     const int bufsize = 1024;
     int retval = 0;
@@ -277,6 +279,12 @@ int is_layer3_qeth(const char *iface)
     ssize_t slen;
     char* driver;
     int fd;
+    char* iface;
+
+    if (interface->parentif)
+        iface = interface->parentif;
+    else
+        iface = interface->name;
 
     // This is sufficient for both /driver and /layer2.
     len = strlen(SYSCLASSNET) + strlen(iface) + strlen("/device/driver") + 1;
@@ -293,11 +301,11 @@ int is_layer3_qeth(const char *iface)
         goto out;
     }
 
-    buf[slen + 1] = '\0';
+    buf[slen] = '\0';
 
     driver = strrchr(buf, '/') + 1;
     if (strcmp(driver, "qeth") != 0) {
-        di_error("no qeth found: %s", driver);
+        di_info("no qeth found: %s", driver);
         goto out;
     }
 
@@ -329,7 +337,7 @@ out:
     return retval;
 }
 #else
-int is_layer3_qeth(const char *iface __attribute__((unused)))
+int is_layer3_qeth(const struct netcfg_interface *interface  __attribute__((unused)))
 {
     return 0;
 }
@@ -408,12 +416,13 @@ int get_all_ifs (int all, char*** ptr)
     char ibuf[512];
     char** list = NULL;
     size_t len = 0;
+    ibuf[sizeof(ibuf)-1]='\0';
 
     if (getifaddrs(&ifap) == -1)
         return 0;
 
     for (ifa = ifap; ifa; ifa = ifa->ifa_next) {
-        strncpy(ibuf, ifa->ifa_name, sizeof(ibuf));
+        strncpy(ibuf, ifa->ifa_name, sizeof(ibuf)-1);
         if (ifa->ifa_flags & IFF_LOOPBACK)   /* ignore loopback devices */
             continue;
 #if defined(__linux__)
@@ -558,6 +567,16 @@ char *get_ifdsc(struct debconfclient *client, const char *if_name)
                     CMD_SUCCESS && client->value != NULL) {
                 return strdup(client->value);
             }
+	    /* If using systemd-udevd stable names try that as well
+	       Those have two character prefix, then one character for
+	       subtype. */
+	    if (strlen(template) > 19) {
+		template[19] = '\0';
+		if (debconf_metaget(client, template, "description") ==
+                    CMD_SUCCESS && client->value != NULL) {
+		    return strdup(client->value);
+		}
+	    }
         } else {
             strcpy(template, "netcfg/internal-wifi");
             debconf_metaget(client, template, "description");
@@ -1116,8 +1135,8 @@ void netcfg_write_loopback (void)
     lo.name = LO_IF;
     lo.loopback = 1;
     
-    netcfg_write_interface(NULL);
-    netcfg_write_interface(&lo);
+    netcfg_write_interface(NULL, NULL, NULL);
+    netcfg_write_interface(NULL, &lo, NULL);
 }
 
 /*
@@ -1310,11 +1329,12 @@ void interface_up (const char *if_name)
 {
     struct ifreq ifr;
 
-    strncpy(ifr.ifr_name, if_name, IFNAMSIZ);
+    strncpy(ifr.ifr_name, if_name, IFNAMSIZ-1);
+    ifr.ifr_name[IFNAMSIZ-1]='\0';
 
     if (skfd && ioctl(skfd, SIOCGIFFLAGS, &ifr) >= 0) {
         di_info("Activating interface %s", if_name);
-        strncpy(ifr.ifr_name, if_name, IFNAMSIZ);
+        strncpy(ifr.ifr_name, if_name, IFNAMSIZ-1);
         ifr.ifr_flags |= (IFF_UP | IFF_RUNNING);
         ioctl(skfd, SIOCSIFFLAGS, &ifr);
     } else {
@@ -1326,11 +1346,12 @@ void interface_down (const char *if_name)
 {
     struct ifreq ifr;
 
-    strncpy(ifr.ifr_name, if_name, IFNAMSIZ);
+    strncpy(ifr.ifr_name, if_name, IFNAMSIZ-1);
+    ifr.ifr_name[IFNAMSIZ-1]='\0';
 
     if (skfd && ioctl(skfd, SIOCGIFFLAGS, &ifr) >= 0) {
         di_info("Taking down interface %s", if_name);
-        strncpy(ifr.ifr_name, if_name, IFNAMSIZ);
+        strncpy(ifr.ifr_name, if_name, IFNAMSIZ-1);
         ifr.ifr_flags &= ~IFF_UP;
         ioctl(skfd, SIOCSIFFLAGS, &ifr);
     } else {
@@ -1338,6 +1359,24 @@ void interface_down (const char *if_name)
     }
 }
 
+void netcfg_interface_up (const struct netcfg_interface *iface)
+{
+	if (iface->parentif)
+		interface_up (iface->parentif);
+
+	if (iface->name)
+		interface_up (iface->name);
+}
+
+void netcfg_interface_down (const struct netcfg_interface *iface)
+{
+	if (iface->name)
+		interface_down (iface->name);
+
+	if (iface->parentif)
+		interface_down (iface->parentif);
+}
+
 void parse_args (int argc, char ** argv)
 {
     if (argc == 2) {
@@ -1361,11 +1400,15 @@ void reap_old_files (void)
 {
     static char* remove[] =
         { INTERFACES_FILE, HOSTS_FILE, HOSTNAME_FILE, NETWORKS_FILE,
-          RESOLV_FILE, DHCLIENT_CONF, DOMAIN_FILE, 0 };
+          DHCLIENT_CONF, DOMAIN_FILE, 0 };
     char **ptr = remove;
 
     while (*ptr)
         unlink(*ptr++);
+
+    if (!is_resolvconf_used())
+	    if (!is_resolved_used())
+		unlink(RESOLV_FILE);
 }
 
 /* Convert a space-separated list of nameservers in a single string (as might
@@ -1528,7 +1571,7 @@ int netcfg_detect_link(struct debconfclient *client, const struct netcfg_interfa
         if (ethtool_lite(if_name) == 1) /* ethtool-lite's CONNECTED */ {
             di_info("Found link on %s", if_name);
 
-            if (!empty_str(gateway) && !is_wireless_iface(if_name) && !is_layer3_qeth(if_name)) {
+            if (!empty_str(gateway) && !is_wireless_iface(if_name) && !is_layer3_qeth(interface)) {
                 for (count = 0; count < gw_tries; count++) {
                     if (di_exec_shell_log(arping) == 0)
                         break;
@@ -1564,6 +1607,8 @@ void netcfg_interface_init(struct netcfg_interface *iface)
     iface->v6_stateless_config = -1;
     iface->loopback = -1;
     iface->mode = MANAGED;
+    iface->parentif = NULL;
+    iface->vlanid = -1;
 }
 
 /* Parse an IP address (v4 or v6), with optional CIDR netmask, into
@@ -1577,10 +1622,12 @@ int netcfg_parse_cidr_address(const char *address, struct netcfg_interface *inte
     struct in_addr addr;
     struct in6_addr addr6;
     int ok;
-    char *maskptr, addrstr[NETCFG_ADDRSTRLEN];
+    char *maskptr, *addrstr, addrbuf[NETCFG_ADDRSTRLEN];
     int i;
     
-    strncpy(addrstr, address, NETCFG_ADDRSTRLEN);
+    strncpy(addrbuf, address, NETCFG_ADDRSTRLEN-1);
+    addrbuf[NETCFG_ADDRSTRLEN-1]='\0';
+    addrstr = strtrim(addrbuf);
     
     if ((maskptr = strchr(addrstr, '/'))) {
         /* Houston, we have a netmask; split it into bits */
@@ -1739,7 +1786,54 @@ void rtrim(char *s)
 	
 	n = strlen(s) - 1;
 	
-	while (isspace(s[n])) {
+	while (n >= 0 && isspace(s[n])) {
 		s[n] = '\0';
+		n--;
 	}
 }
+
+char *strtrim(char *s)
+{
+	size_t len;
+
+	len = strlen(s);
+	if (!len)
+		return s;
+
+	rtrim(s);
+
+	while (*s && isspace(*s))
+		s++;
+
+	return s;
+}
+
+/* Check if /etc/resolv.conf symlinks to /run/resolvconf/resolv.conf. */
+int is_resolvconf_used()
+{
+	int resolvconf_used = 0;
+
+	char *path = realpath(RESOLV_FILE, NULL);
+
+	if (path) {
+		resolvconf_used = !strcmp(path, RESOLVCONF_FILE);
+		free(path);
+	}
+
+	return resolvconf_used;
+}
+
+/* Check if /etc/resolv.conf symlinks to /run/systemd/resolve/stub-resolv.conf. */
+int is_resolved_used()
+{
+	int resolved_used = 0;
+
+	char *path = realpath(RESOLV_FILE, NULL);
+
+	if (path) {
+		resolved_used = !strcmp(path, RESOLVED_FILE);
+		free(path);
+	}
+
+	return resolved_used;
+}
