@@ -101,8 +101,8 @@ class NetworkStoreTests(unittest.TestCase):
         self.model.add_device('/foo', 'Intel', 'Wireless')
         self.model.add_device('/bar', 'Intel', 'Wireless')
 
-        self.assert_(self.model.has_device('/foo'))
-        self.assert_(not self.model.has_device('/not-there'))
+        self.assertTrue(self.model.has_device('/foo'))
+        self.assertTrue(not self.model.has_device('/not-there'))
 
     def test_get_device_ids(self):
         self.model.add_device('/foo', 'Intel', 'Wireless')
@@ -118,24 +118,24 @@ class NetworkStoreTests(unittest.TestCase):
         device_it = self.model.get_iter_first()
 
         ap_it = self.model.iter_children(device_it)
-        self.assert_(ap_it)
+        self.assertTrue(ap_it)
         self.assertEqual(self.model.get(ap_it, 0, 1, 2), ('Orange', True, 40))
 
         ap_it = self.model.iter_next(ap_it)
-        self.assert_(ap_it)
+        self.assertTrue(ap_it)
         self.assertEqual(self.model.get(ap_it, 0, 1, 2), ('Apple', False, 60))
 
         ap_it = self.model.iter_next(ap_it)
-        self.assert_(not ap_it)
+        self.assertTrue(not ap_it)
 
     def test_has_ap(self):
         self.model.add_device('/foo', 'Intel', 'Wireless')
         self.model.add_ap('/foo', 'Orange', True, 40)
         self.model.add_ap('/foo', 'Apple', False, 60)
 
-        self.assert_(self.model.has_ap('/foo', 'Orange'))
-        self.assert_(not self.model.has_ap('/not-there', 'Orange'))
-        self.assert_(not self.model.has_ap('/foo', 'Aubergine'))
+        self.assertTrue(self.model.has_ap('/foo', 'Orange'))
+        self.assertTrue(not self.model.has_ap('/not-there', 'Orange'))
+        self.assertTrue(not self.model.has_ap('/foo', 'Aubergine'))
 
     def test_set_ap_strength(self):
         self.model.add_device('/foo', 'Intel', 'Wireless')
